<?php

/**
 * FaceBookPixel short summary.
 *
 * FaceBookPixel description.
 *
 * @version 1.0
 * @author Jacob.Hulse
 * @comapy Knight Shift Productions
 */
class KSPFaceBookPixel
{

    /**
     * Pixel identifier provided by Facebook on generation
     * @var string
     */
    private $PixelID = '';



    public function __construct() {
        add_action('wp_head',               array($this, 'insert_Main_Pixel'));
        add_action('wp_footer',             array($this, 'generate_Page_Events'));
        add_action('wp_footer',             array($this, 'generate_WooCommerce_Events'));
        add_action('wp_enqueue_scripts',    array($this, 'Enqueue_Scripts'));
    }


    

    /**
     * Registers scripts used by this plugin on the front end only.
     * 
     * Hooked into the 'wp_enqueue_scripts' action.
     */
    function Enqueue_Scripts()
    {
        if (!is_admin()) {
            wp_deregister_script('jquery');  
            wp_register_script('jquery', 'https://ajax.googleapis.com/ajax/libs/jquery/2.1.4/jquery.min.js', FALSE, '2.1.4', TRUE);  
            wp_enqueue_script('jquery');

            wp_register_script('ksputilities.js', plugins_url('../inc/scripts/ksputilities.js', __FILE__), false, null, true);
        }
    }


    /**
     * Indicates if we have a value in the Pixel Code
     * 
     * Returns True if we do
     * Returns False if no Pixel code was set
     * @return bool
     */
    private function havePixel() {
        $pixel = ksp_wp_facebook_pixel_get_plugin_option('facebook_pixel');
        if (empty($pixel)) return false;
        return true;
    }


    public function insert_Main_Pixel() {


        //Base Pixel from Plugin Settings
        $pixel = ksp_wp_facebook_pixel_get_plugin_option('facebook_pixel');
        if (empty($pixel)) return; //We got nothing Jim

        //Get our PixelID
        $re = "/fbq\\s*\\(\\s*('|\")init('|\")\\s*,\\s*('|\")(.*?)('|\")\\s*\\)\\s*;/"; 
        preg_match_all($re, $pixel, $matches);
        $this->PixelID = $matches[4][0];
        
        echo $pixel;
    }


    public function generate_WooCommerce_Events() {
        
        if(class_exists('WooCommerce') && $this->havePixel()) {
            
            $script = '<script type="text/javascript">//Add WooCommerce Events
                window.onload = function() {';
            $noscript = '<noscript>';

            //Product Page: View Content & Add To Cart
            if (is_product()) {
                global $product;
                $event_data = array(
                    array('key' => 'content_type', 'value' => 'product'),
                    array('key' => 'content_ids', 'value' => $product->id),
                    array('key' => 'value', 'value' => $product->get_price()),
                    array('key' => 'currency', 'value' => get_woocommerce_currency()),
                );
                $event = $this->generate_Page_Event('ViewContent', $event_data);
                $script .= $event['script'];
                $noscript .= $event['image']."\r\n";
                $event = $this->generate_Page_Event('AddToCart', $event_data, '[class*=add_to_cart_button]', 'click', true);
                $script .= $event['script'];
                $noscript .= $event['image']."\r\n";


            }


            //Main Shop: Add To Cart
            if (is_shop()) {
                $event_data = array(
                    'content_type' => array('key' => 'content_type', 'value' => 'product'),
                    'content_ids' => array('key' => 'content_ids', 'value' => '~content_ids~'),
                    'value' => array('key' => 'value', 'value' => '~value~'),
                    'currency' => array('key' => 'currency', 'value' => get_woocommerce_currency()),
                );


                $event = $this->generate_Page_Event('AddToCart', $event_data, '[class*=add_to_cart_button]', 'click');
                $script .= $event['script'];
                $noscript .= $event['image']."\r\n";
            }


            
            //Checking out: Initiate Purchase
            //is_checkout()


            //Order Complete: Purchase
            if(is_wc_endpoint_url('order-received')) {
                //global $woocommerce;
                //$cart = $woocommerce->cart;
                
                $content_ids = array();
                $options = ksp_wp_facebook_pixel_get_plugin_option('woocommerce_options' )[0];

                global $wp;
                $order_id = $wp->query_vars['order-received'];

                $order = new WC_Order( apply_filters( 'woocommerce_thankyou_order_id', absint( $order_id ) ) );
                $products = $order->get_items();
                $count = 0;

                $productFactory = new WC_Product_Factory();

                foreach ($products as $product)
                {
                    //Get Product IDs
                    switch ($options['content_id'])
                    {
                        case 'data-product_id':
                            $content_ids[] = $product['product_id'];
                            break;

                        case 'data-product_sku':
                            $_product = $productFactory->get_product($product['product_id']);
                            $content_ids[] = $_product->get_sku();
                            break;
                    }
                    $count += empty( $product['qty'] ) ? 1 : $product['qty'];

                }
                
                $event_data = array(
                    array('key' => 'content_type', 'value' => 'product'),
                    array('key' => 'content_ids', 'value' => $content_ids),
                    array('key' => 'value', 'value' => strip_tags(str_replace(get_woocommerce_currency_symbol(), '', $order->get_total()))),
                    array('key' => 'currency', 'value' => get_woocommerce_currency()),
                    array('key' => 'num_items', 'value' => $count )
                );
                $event = $this->generate_Page_Event('Purchase', $event_data);
                $script .= $event['script'];
                $noscript .= $event['image']."\r\n";


            }

            $script .= '}</script>';
            $noscript .= '</noscript>';
            echo $script;
            echo $noscript;
        }
    }


    public function generate_Page_Events() {
        $prefix = PLUGIN_ID.'_metabox_';

        //Get Post Specific Pixel
        $page_event = get_post_meta( get_the_ID(), $prefix.'event', true );
        if (empty($page_event)) return; //We got nothing Jim

        /* 
         * Event Data JSON
         * JSON Update Code
         * Event Function Call
         * Event Image Tag (if no variable values)
         */
        $page_events = array();

        //Place in loop for multiple events on single page
        $event_data = get_post_meta( get_the_ID(), $prefix.'event_values', true );
        $page_events[] = $this->generate_Page_Event($page_event, $event_data);
        //End Loop
        
        wp_enqueue_script('ksputilities.js');
        $script = '<script type="text/javascript">//Add Standard Events
                window.onload = function() {
                ';
        $noscript = '<noscript>';
        foreach ($page_events as $event)
        {
        	$script .= 
                     $event['jsonData']."\r\n"
                    .$event['jsonDataUpdate']."\r\n"
                    .$event['fbq']."\r\n";
            $noscript .= $event['image']."\r\n";
        }
        $script .= '}</script>';
        $noscript .= '</noscript>';
        echo $script;
        echo $noscript;

    }


    private function generate_Page_Event($page_event, $event_data, $targetSelector = null, $targetAction = null, $preventDefault = false) {
        $event = array();
        $event_trigger = '';
        $event_json = '';
        $event_json_update = '';
        $event_fbq = '';
        $event_image = '';
        $GetImage = true;
        $image_data = '';
        

        //Post specific event key/value pairs
        if(!empty($event_data)) {
            
            if(!empty($targetSelector) && !empty($targetAction)) {
                $event_trigger = '
                    jQuery("'.$targetSelector.'").one("'.$targetAction.'", function(e) {';
                $GetImage = false;
                if ($preventDefault) {
                    $event_trigger .= '
                        e.preventDefault();';
                }

            } else {
                $event_trigger = 'jQuery(document).ready(function (e) {';
            }


            $event_json = 'eventJSON = {';            
            /*  REG EX TESTER
             *  https://regex101.com/r/wB4bK6/1
             * */
            foreach( $event_data as $event_keyvalue_pair ) {
                $key = $event_keyvalue_pair['key'];
                $value = $event_keyvalue_pair['value'];

                //Replace Query string values
                if (preg_match('/\\?.*\\?/', $value)) {
                    $event_json_update .= 'eventJSON.'.$key.' = getQueryStringValue("'.str_replace('?', '', $value).'");';
                    $value = '';
                    $GetImage = false;
                }

                //Replace Javascript Variable values
                if (preg_match('/%.*%/', $value)) {
                    $event_json_update .= 'eventJSON.'.$key.' = '.str_replace('%', '', $value).';';
                    $value = '';
                    $GetImage = false;
                }


                //Update with triggering objects Attribute value
                if (preg_match("/[[][^\"'].*[]][^\"']/", $value)) {
                    $event_json_update .= 'eventJSON.'.$key.' = jQuery(this).attr("'.str_replace('[', '', str_replace(']', '', $value)).'");';
                    $value = '';
                    $GetImage = false;
                }


                //Update WooCommerce Values
                if (preg_match('/[~].*[~]/', $value)) {
                    $value = str_replace('~', '', str_replace('~', '', $value));
                    $options = ksp_wp_facebook_pixel_get_plugin_option('woocommerce_options' )[0];

                    switch ($value) {
                        case 'content_ids':
                            $event_json_update .= '
                                eventJSON.'.$key.' = jQuery(this).attr("'.$options['content_id'].'");';
                            break;

                        case 'value':
                            //DOES NOT WORK FOR SINGLE PRODUCT PAGE>>>FIX THIS
                            $event_json_update .= '
                                eventJSON.'.$key.' = jQuery(this).parent().find(".price").text().replace("$", "");
                                if (eventJSON.'.$key.' === "") {eventJSON.'.$key.' = jQuery(this).parent().parent().find("[itemprop=\'price\']").attr("content"); }
                            ';
                            break;
                    }

                    $value = '';
                    $GetImage = false;
                }

                $event_json .= $key.":".kspu_QuoteString($value).",";
                $image_data .= '&amp;cd['.$key.']='.$value;
            }

            $event_json = rtrim($event_json, ",").'};';
        }
            
        $event_fbq = "fbq('track', '".$page_event."'";
        if(empty($event_json)) { 
            $event_fbq .= ");";

        } else {
            $event_fbq .= ", eventJSON);";

        }
        
        if($preventDefault) {
            $event_fbq .= '
                e.isDefaultPrevented = function(){ return false; };
                jQuery(e.target).trigger(e); //return false;';
        }

        if($GetImage) { 
            //Generate img tag
            /*
             * <img height="1" width="1" style="display:none" src="https://www.facebook.com/tr?id=FB_PIXEL_ID&amp;ev=Purchase
             *      &amp;cd[currency]=EUR
             *      &amp;cd[value]=15.20" />
             */

            $event_image = '<img height="1" width="1" style="display:none" src="https://www.facebook.com/tr?id='.$this->PixelID.'&amp;ev='.$page_event.$image_data.'" />';
        }

        $event['trigger'] = $event_trigger;
        $event['jsonData'] = $event_json;
        $event['jsonDataUpdate'] = $event_json_update;
        $event['fbq'] = $event_fbq;
        $event['script'] = '
                    '.$event['trigger'].'
                    '.$event['jsonData'].'
                    '.$event['jsonDataUpdate'].'
                    '.$event['fbq'].'
                });';
        $event['image'] = $event_image;
        return $event;

    }




}
$kspFBPixel = new KSPFaceBookPixel();

