<?php
/**
 * Plugin Name: WP Facebook Pixel
 * Plugin URI: http://knightshiftproductions.azurewebsites.net/
 * Description: Adds the new Facebook Pixel to all pages automatically and allows you to add standard Facebook events on each page.
 * Version: 2.1
 * Author: Knight Shift Productions
 * Author URI: http://knightshiftproductions.azurewebsites.net/
 * License GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Requires at least: 4.0
 * Tested up to: 4.4.1
 */


// For Version Names: http://www.dogbreedinfo.com/purebred.htm



/**
 * Silence is golden; exit if accessed directly
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}




/**
 * Define Plugin Constants
 */
define('PLUGIN_ID', 'ksp_wp_facebook_pixel', true);
define('PLUGIN_NAME', 'WP Facebook Pixel', true);
define('PLUGIN_VERSION', '2.0', true);
define('PLUGIN_OPTIONS', PLUGIN_ID.'_options');



register_activation_hook( __FILE__, 'ksp_wp_facebook_pixel_plugin_on_activate' );
register_deactivation_hook( __FILE__, 'ksp_wp_facebook_pixel_plugin_on_deactivation' );
register_uninstall_hook( __FILE__, 'ksp_wp_facebook_pixel_plugin_on_uninstall' );
add_filter( 'plugin_action_links', 'ksp_wp_facebook_pixel_add_action_plugin', 10, 5 );      //Adds special links to Plugin's page
add_action('admin_enqueue_scripts', 'kspwpfbp_admin_enqueue_scripts');
add_action('wp_enqueue_scripts', 'kspwpfbp_enqueue_scripts');






/**
 * Bootstrap CMB2
 * No need to check versions or if CMB2 is already loaded... the init file does that already!
 *
 * Check to see if CMB2 exists, and either bootstrap it or add a notice that it is missing
 */
if ( file_exists( dirname( __FILE__ ) . '/inc/CMB2/init.php' ) ) {
	require_once 'inc/CMB2/init.php';
} else {
	add_action( 'admin_notices', 'cmb2_example_plugin_missing_cmb2' );
}


/**
 * Add an error notice to the dashboard if CMB2 is missing from the plugin
 *
 * @return void
 */
function cmb2_example_plugin_missing_cmb2() { ?>
    <div class="error">
	    <p><?php _e( 'CMB2 Example Plugin is missing CMB2!', 'cmb2-example-plugin' ); ?></p>
    </div>
<?php }




/**
 * Send Email to me when the plugin is activated.
 */
function ksp_wp_facebook_pixel_plugin_on_activate() {
}


function ksp_wp_facebook_pixel_plugin_on_deactivation() {
}

function ksp_wp_facebook_pixel_plugin_on_uninstall() {
}




/**
 * Adds actions to Plugin listing on Plugins page.
 * 
 * Hooked to the 'plugin_action_links' filter.
 * @param array $actions 
 * @param string $plugin_file 
 * @return array
 */
function ksp_wp_facebook_pixel_add_action_plugin( $actions, $plugin_file ) 
{
	static $plugin;

	if (!isset($plugin))
		$plugin = plugin_basename(__FILE__);

	if ($plugin == $plugin_file) {
        $settings = array('settings' => '<a href="options-general.php?page='.PLUGIN_OPTIONS.'">' . __('Settings', 'General') . '</a>');
        $actions = array_merge($settings, $actions);
    }
    
    return $actions;
}




/**
 * Load plugin resources
 */
require_once 'inc/KSPUtilities.php';        //Always include this first
require_once 'framework/PluginOptions.php';
require_once 'framework/MetaBox.php';
require_once 'assets/KSPFaceBookPixel.php';    //Where all the magic happens
function kspwpfbp_admin_enqueue_scripts() {
    if (is_admin())   
    {  
        wp_enqueue_style('KSPAdmin', plugins_url('/inc/css/KSPAdmin.css', __FILE__), false, false, false);
    } 
}
function kspwpfbp_enqueue_scripts() {
    if (!is_admin()) {
        wp_enqueue_script('jquery');
    }

}







